// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO24/DIO24_Linux_8.x.x.x_GSC_DN/driver/device.c $
// $Rev: 52088 $
// $Date: 2022-11-23 10:57:37 -0600 (Wed, 23 Nov 2022) $

// DIO24: Device Driver: source file

#include "main.h"



// variables ******************************************************************

const gsc_dev_id_t	dev_id_list[]	=
{
	// model	Vendor	Device	SubVen	SubDev	type
	{ "DIO24",	0x10B5, 0x9080, 0x10B5, 0x2706,	GSC_DEV_TYPE_DIO24	},
	{ NULL }
};



//*****************************************************************************
static void _clock_counter_compute(dev_data_t* dev)
{
	dev->cache.clock_counter	= ((dev->cache.frr & 0xFF) >= 0x03) ? 1 : 0;
}



//*****************************************************************************
static void _di_compute(dev_data_t* dev)
{
	dev->cache.dedicated_input	= 1;
}



//*****************************************************************************
static void _irq_compute(dev_data_t* dev)
{
	dev->cache.irq_qty	= ((dev->cache.frr & 0xFF) >= 0x02) ? 48 : 0;
}



//*****************************************************************************
static void _jumpers_compute(dev_data_t* dev)
{
	dev->cache.jumper_val	= (dev->cache.frr >> 16) & 0x3;
	dev->cache.jumper_qty	= 2;
	dev->cache.jumper_sense	= 0;
}



//*****************************************************************************
static void _proc_id_str_compute(dev_data_t* dev)
{
	if (dev->cache.jumper_qty)
		sprintf(dev->proc_id_str, "0x%lX", (long) dev->cache.jumper_val);
	else
		strcpy(dev->proc_id_str, "none");
}



/******************************************************************************
*
*	Function:	dev_device_create
*
*	Purpose:
*
*		Do everything needed to setup and use the given device.
*
*	Arguments:
*
*		dev		The structure to initialize.
*
*	Returned:
*
*		0		All is well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int dev_device_create(dev_data_t* dev)
{
	static const gsc_bar_maps_t	bar_map	=
	{
		{
			// mem	io	rw
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 0: PLX registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 1: PLX registers, I/O mapped
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 2: GSC registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 3: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 4: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO }	// BAR 5: unused
		}
	};

	int	ret;

	for (;;)	// A convenience loop.
	{
		// Verify some macro contents.
		ret	= gsc_macro_test_base_name(DIO24_BASE_NAME);
		if (ret)	break;

		ret	= gsc_macro_test_model();
		if (ret)	break;

		// PCI setup.
		ret	= os_pci_dev_enable(&dev->pci);
		if (ret)	break;

		ret	= os_pci_master_set(&dev->pci);
		if (ret)	break;

		// Control ISR access to the device and data structure.
		ret	= os_spinlock_create(&dev->spinlock);
		if (ret)	break;

		// Control access to the device and data structure.
		ret	= os_sem_create(&dev->sem);
		if (ret)	break;

		// Access the BAR regions.
		ret	= gsc_bar_create(dev, &dev->bar, &bar_map);
		if (ret)	break;

		// Firmware access.
		dev->vaddr.gsc_bcr_32			= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_BCR));
		dev->vaddr.gsc_bsr_32			= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_BSR));

		dev->vaddr.gsc_ccr_32			= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_CCR));
		dev->vaddr.gsc_cclfixr_32[0]	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_CCLFI0R));
		dev->vaddr.gsc_cclfixr_32[1]	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_CCLFI1R));
		dev->vaddr.gsc_cclfixr_32[2]	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_CCLFI2R));
		dev->vaddr.gsc_cclfixr_32[3]	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_CCLFI3R));
		dev->vaddr.gsc_cclrixr_32[0]	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_CCLRI0R));
		dev->vaddr.gsc_cclrixr_32[1]	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_CCLRI1R));
		dev->vaddr.gsc_cclrixr_32[2]	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_CCLRI2R));
		dev->vaddr.gsc_cclrixr_32[3]	= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_CCLRI3R));

		dev->vaddr.gsc_ddir_32			= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_DDIR));
		dev->vaddr.gsc_ddor_32			= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_DDOR));

		dev->vaddr.gsc_frr_32			= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_FRR));

		dev->vaddr.gsc_ifcr_32			= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_IFCR));
		dev->vaddr.gsc_ifsr_32			= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_IFSR));
		dev->vaddr.gsc_iocr_32			= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_IOCR));
		dev->vaddr.gsc_ircr_32			= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_IRCR));
		dev->vaddr.gsc_irsr_32			= GSC_VADDR(dev, GSC_REG_OFFSET(DIO24_GSC_IRSR));

		// Initialize the cache.

		dev->cache.frr					= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_frr_32);

		_clock_counter_compute(dev);
		_di_compute(dev);
		_irq_compute(dev);
		_jumpers_compute(dev);
		_proc_id_str_compute(dev);	// must be after jumpers

		// Initialize additional resources.
		ret	= dev_irq_create(dev);

		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_device_destroy
*
*	Purpose:
*
*		Do everything needed to release the referenced device.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_device_destroy(dev_data_t* dev)
{
	if (dev)
	{
		dev_irq_destroy(dev);
		gsc_bar_destroy(&dev->bar);
		os_sem_destroy(&dev->sem);
		os_spinlock_destroy(&dev->spinlock);
		os_pci_master_clear(&dev->pci);
		os_pci_dev_disable(&dev->pci);
	}
}


